@@ -6,8 +6,7 @@
 PqDataFrame::PqDataFrame(PqResultSource* result_source,
                          const std::vector<std::string>& names,
                          const int n_max_,
-                         const std::vector<DATA_TYPE>& types,
-                         std::vector<Oid> oids) :
+                         const std::vector<DATA_TYPE>& types) :
   DbDataFrame(new PqColumnDataSourceFactory(result_source, types), names, n_max_, types)
 {
 }
